"""Main entrypoint for the Flask application."""

from flask import Flask, render_template, redirect, request, url_for
from flask_caching import Cache
from scraper import FTPScraper
import models

# pylint: disable=invalid-name
app = Flask(__name__)
cache = Cache(app, config={'CACHE_TYPE': 'simple'})

DISPLAY_DATETIME_FORMAT = '%A, %B %d, %Y at %-I:%M %p'
DISPLAY_DATE_FORMAT = '%A, %B %-d, %Y'
DISPLAY_TIME_FORMAT = '%-I:%M %p on %A'


@app.after_request
def add_header(response):
    """Add an HTTP response header for cache invalidation."""
    response.cache_control.max_age = 300
    return response


@app.template_filter('strftime')
def _jinja2_filter_datetime(datetime, date=True, time=True):
    return datetime.strftime(DISPLAY_DATETIME_FORMAT if date and time
                             else DISPLAY_DATE_FORMAT if date
                             else DISPLAY_TIME_FORMAT if time else '')


@app.route("/")
def index():
    """Render the app landing page."""
    return render_template('index.html')


def _make_cache_key():
    """Create a cache key for Flask-Caching."""
    path = request.path
    args = str(hash(frozenset(request.args.items())))
    return (path + args).encode('utf-8')


@app.route("/live")
@cache.cached(timeout=300, key_prefix=_make_cache_key)
def live(results_url=None):
    """Render the primary view of live tournament stats."""
    results_url = request.args.get('results_url')
    if not results_url:
        return redirect(url_for('index'))

    scraper = FTPScraper(results_url)
    tournament = scraper.scrape()
    return render_template('live.html', tournament=tournament, events=tournament.events,
                           phases=models.EventPhase)


if __name__ == "__main__":
    app.run()
