"""Extract fencing event registration stats from Fencing Time webpages."""

import asyncio
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from urllib.parse import urlparse, urljoin
from lxml import html  # nosec Bandit suggests defusedxml but defusedxml.lxml is dead
import requests
from models import Event, EventPhase, Fencer, Tournament


class FTPScraper:
    """Scraper for tournaments hosted on an FTP server.

    This reads the original Fencing Time results pages, hosted by individual
    tournament operators.
    """

    UPDATED_DATETIME_FORMAT = '%m/%d/%Y - %I:%M %p'
    EVENT_DATETIME_FORMAT = '%A, %B %d, %Y - %I:%M %p'

    def __init__(self, tournament_url):
        """Set up the scraper instance."""
        self.tournament_url = tournament_url
        self.tournament = None

    def scrape(self):
        """Get all tournament information."""
        try:
            results = requests.get(self.tournament_url)
        except requests.exceptions.MissingSchema:
            results = requests.get("http://{}".format(self.tournament_url))
        results_tree = html.fromstring(results.content)
        try:
            tournament_name = results_tree.xpath('//span[@class="tournName"]/text()')[0]
            updated_str = (results_tree.xpath('//span[@class="lastUpdate"]/text()')[0]
                           .replace('Last Updated:', '').strip())
            updated = datetime.strptime(updated_str, self.UPDATED_DATETIME_FORMAT)
        except IndexError:
            raise ScrapeError("Tournament info not found.")

        self.tournament = Tournament(name=tournament_name, url=results.url,
                                     updated=updated)

        # Get tournament events
        try:
            event_urls = results_tree.xpath(
                '//div[@id="schedule"]/table/tr/td/a[text()="View"]/@href')
        except IndexError:
            raise ScrapeError("No event schedule found.")

        loop = asyncio.new_event_loop()
        loop.run_until_complete(self._scrape_events(event_urls))

        return self.tournament

    async def _scrape_events(self, event_urls):
        """Get event information asynchronously."""
        with ThreadPoolExecutor(max_workers=20) as executor:
            loop = asyncio.get_event_loop()
            futures = []

            for event_url in event_urls:
                if not urlparse(event_url).netloc:
                    event_url = urljoin(self.tournament.url, event_url)
                futures.append(loop.run_in_executor(executor, requests.get, event_url))

            for response in await asyncio.gather(*futures):
                event = self.parse_event(response)
                self.tournament.events.append(event)

            self.tournament.count_fencers()

    def parse_event(self, event):
        """Extract useful strings from the event info."""
        event_tree = html.fromstring(event.content)
        event_details = event_tree.xpath('//span[@class="tournDetails"]/text()')
        try:
            event_name = event_details[0]
            event_time_str = event_details[1]
            event_time = datetime.strptime(event_time_str, self.EVENT_DATETIME_FORMAT)
        except IndexError:
            raise ScrapeError("Failed to interpret live results for event \"{}\"."
                              .format(event_details))

        # Get the event status
        if event_tree.xpath('//a[text()="Final Results"]'):
            fencers = event_tree.xpath(
                '//div[@id="finalResults"]/table/tr/td[2]/text()')
            fencers = [Fencer(f, True) for f in fencers]
            event_phase = EventPhase.FINISHED
        elif event_tree.xpath('//a[text()="Seeding"]'):
            fencers = event_tree.xpath(
                '//div[@id="Round1Seeding"]/table/tr/td[2]/text()')
            fencers = [Fencer(f, True) for f in fencers]
            event_phase = EventPhase.STARTED
        elif event_tree.xpath('//a[text()="Check-In Status"]'):
            event_phase = EventPhase.REGISTRATION
            fencers_checked_in = [
                bool(list(f))
                for f in event_tree.xpath('//div[@id="checkIn"]/table/tr/td[1]')]
            fencers = event_tree.xpath('//div[@id="checkIn"]/table/tr/td[2]/text()')
            fencers = [Fencer(f, ci) for (f, ci) in zip(fencers, fencers_checked_in)]

        return Event(name=event_name, time=event_time, phase=event_phase,
                     url=event.url, fencers=fencers, tournament=self.tournament)


class FTLiveScraper(FTPScraper):
    """Scraper for tournaments hosted on fencingtimelive.com.

    This reads the newer-style pages, centrally hosted by Fencing Time.
    """
    # to do...


class ScrapeError(Exception):
    """A problem with scraping."""
